#ifndef __FAR_FTP_PLUGIN
#define __FAR_FTP_PLUGIN

#include <FARStdlib/pack1.h>

//------------------------------------------------------------------------
// TYPES
//------------------------------------------------------------------------
#if !defined(__FAR_PLUGIN_FTP)
  PRESTRUCT( FTPInterface );
  PRESTRUCT( FTPPluginInterface );

  //After FTP call to QueryInterface function this variable is set to
  //  valid FTP interface object.
  extern PFTPInterface FTP_Info;

  // Each plugin must declare this function and include "p_All.cpp" file
  extern PFTPPluginInterface FP_DECLSPEC FTPPluginGetInterface( void );

  // Each plugin must declare this function. Even if you set /NoEntry option
  extern BOOL FP_DECLSPEC FTP_PluginStartup( DWORD Reason );

  //Helpers
  #undef PROC
  #undef Log
  #if defined(__FILELOG__)
    #define PROC(v) PluginINProc _inp( FTP_Info->LogProcStart v );
    #define Log(v)  FTP_Info->SayLog v
  #else
    #define PROC(v)
    #define Log(v)
  #endif

  #undef  THROW_ERROR
  #define THROW_ERROR(err,fl,nm)  FTP_Info->Assertion( "Assertion...\nConditin: \"%s\"\nAt file: \"%s:%d\"",err,fl,nm )
#endif

typedef signed __int64 TRINT;

STRUCTBASE( FTPItemList, public FP_ItemList )
    TRINT TotalFullSize;
    TRINT TotalFiles;
  public:
    FTPItemList( BOOL NeedToDelete = TRUE ) : FP_ItemList(NeedToDelete) {
      TotalFullSize = 0;
      TotalFiles    = 0;
    }
};

enum FTPOptionIndex {
  FTP_OPT_MONTH,
  FTP_OPT_SHOWIDLE,
  FTP_OPT_PROCESSCOLOR,
  FTP_OPT_none
};

#include "../Lib/ftp_Lang.h"      //Language ID`s

//------------------------------------------------------------------------
// FTP INTERFACE
//------------------------------------------------------------------------
#define FTP_INTERFACE_MAGIC   MK_ID( 'F','T','P',1 )

STRUCT( FTPInterface )
   DWORD  Magic;
   DWORD  SizeOf;

   PPluginStartupInfo     Info;
   PFarStandardFunctions  FSF;
   char                  *PluginRootKey;
   char                  *PluginStartPath;
   OSVERSIONINFO         *WinVer;
   HMODULE                FTPModule;

//FAR
   CONSTSTR (FP_DECLSPEC    *GetMsg)( int MsgNum );
   CONSTSTR (FP_DECLSPEC    *GetMsgStr)( CONSTSTR Msg);

//Debug
   void     (FP_DECLSPEC_PT *Assertion)( CONSTSTR Format,... );
   void     (FP_DECLSPEC_PT *SayLog)( CONSTSTR Format,... );
   HANDLE   (FP_DECLSPEC_PT *LogProcStart)( CONSTSTR FunctionName,CONSTSTR Format,... );
   void     (FP_DECLSPEC    *LogProcEnd)( HANDLE Proc );

//Reg
   int      (FP_DECLSPEC    *GetRegKeyFullInt)( CONSTSTR Key,CONSTSTR ValueName,DWORD Default );
   char*    (FP_DECLSPEC    *GetRegKeyFullStr)( CONSTSTR Key,CONSTSTR ValueName,char *ValueData,char *Default,DWORD DataMaxSize );
   int      (FP_DECLSPEC    *GetRegKeyInt)( CONSTSTR ValueName,DWORD Default );
   char*    (FP_DECLSPEC    *GetRegKeyStr)( CONSTSTR ValueName,char *ValueData,char *Default,DWORD DataMaxSize );

//Std
   LPVOID   (FP_DECLSPEC    *Alloc)( DWORD sz );
   void     (FP_DECLSPEC    *Del)( LPVOID ptr );
   LPVOID   (FP_DECLSPEC    *Realloc)( LPVOID ptr,DWORD sz );
   DWORD    (FP_DECLSPEC    *PtrSize)( LPVOID ptr );
   BOOL     (FP_DECLSPEC    *HeapCheck)( void );

   int      (FP_DECLSPEC    *StrCmp)( CONSTSTR str,CONSTSTR str1,int maxlen = -1, BOOL isCaseSens = TRUE );
   char*    (FP_DECLSPEC    *StrCpy)( char *dest,CONSTSTR src,int dest_sz = -1 );
   char*    (FP_DECLSPEC    *StrCat)( char *dest,CONSTSTR src,int dest_sz = -1 );
   int      (FP_DECLSPEC    *strLen)( CONSTSTR str );

   int      (FP_DECLSPEC_PT *Sprintf)( char *Buff,CONSTSTR Fmt,... );
   int      (FP_DECLSPEC_PT *SNprintf)( char *Buff,size_t cn,CONSTSTR Fmt,... );
   int      (FP_DECLSPEC    *VSprintf)( char *Buff,CONSTSTR Fmt,va_list arglist );
   int      (FP_DECLSPEC    *VSNprintf)( char *Buff,size_t cn,CONSTSTR Fmt,va_list arglist );

//Utilities
   CONSTSTR (FP_DECLSPEC_PT *Message)( CONSTSTR patt,... );
   CONSTSTR (FP_DECLSPEC    *MessageV)( CONSTSTR patt,va_list a );
   char*    (FP_DECLSPEC    *PointToName)(char *Path);
   char*    (FP_DECLSPEC    *FDigit)( char *buff,TRINT Value,int BuffSize /*-1*/ );  // Output digit to string. Delimits thousands.
   CONSTSTR (FP_DECLSPEC    *FCps)( char *buff,double val );                         // Create CPS value string (Allways 3+1+3+1 length)
   int      (FP_DECLSPEC    *FMessage)( unsigned int Flags,const char *HelpTopic,CONSTSTR *Items,int ItemsNumber,int ButtonsNumber );
   BOOL     (FP_DECLSPEC    *CheckForEsc)( BOOL isConnection,BOOL IgnoreSilent = FALSE );
   void     (FP_DECLSPEC    *IdleMessage)( CONSTSTR str,int color );

//FTP related
   int      (FP_DECLSPEC    *FtpGetRetryCount)( HANDLE hConnect );
   int      (FP_DECLSPEC    *FtpCmdBlock)( HANDLE hConnect,int block /*TRUE,FALSE,-1*/ );
   HANDLE   (FP_DECLSPEC    *Option)( FTPOptionIndex Index, DWORD Value = 0 );
};

#if !defined(__FAR_PLUGIN_FTP)
  CLASS( PluginINProc )
    HANDLE Handle;
   public:
     PluginINProc( HANDLE p )   { Handle = p; }
     ~PluginINProc()            { FTP_Info->LogProcEnd( Handle ); }
  };
#endif
//------------------------------------------------------------------------
// Any plugin
//------------------------------------------------------------------------
typedef HANDLE (WINAPI *FTP_CreateObject_t)( void );
typedef void   (WINAPI *FTP_DestroyObject_t)( HANDLE Object );

STRUCT( FTPPluginInterface )
   DWORD Magic;
};

typedef PFTPPluginInterface (WINAPI *FTPQueryInterface_t)( PFTPInterface FTPInfo );

//------------------------------------------------------------------------
// Special plugin interfaces
//------------------------------------------------------------------------
#include "../Lib/fp_Progress.h"

#include <FARStdlib/pop.h>

#endif